SystemOrganization addCategory: #OrionGenerator!
SystemOrganization addCategory: #'OrionGenerator-Test'!

TestCase subclass: #ASGeneratorTest
	instanceVariableNames: 'aGenerator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrionGenerator-Test'!

!ASGeneratorTest methodsFor: 'running' stamp: 'AnneEtien 4/3/2013 17:09'!
setUp
	
	aGenerator := ASGenerator new.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 18:02'!
testCodeForASSelector

	self assert: (aGenerator codeForASSelector: (aGenerator getPropertiesOf: FAMIXTestClass) first) equals: 'kindAS: aVersion
	^ self privateKind versionIn: aVersion'! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 18:02'!
testCodeForSelector

	self assert: (aGenerator codeForSelector: (aGenerator getPropertiesOf: FAMIXTestClass ) first) equals: 'kind
	^ self kindAS: self currentContext'! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:03'!
testCreateASClass
	
	|  anASClass nbClass |
	
	nbClass := Smalltalk allClasses size.
	
	anASClass := aGenerator createASClass: FAMIXTestClass.

	self assert: anASClass notNil.
	self assert: (anASClass name) equals: #ASTestClass.
	self assert: (anASClass traitComposition allTraits contains: [:each | each name = #TFAMIXTestClass]).
	self assert: (anASClass superclass) equals: FAMIXTestClass.
	self assert: Smalltalk allClasses size equals: nbClass + 1.
	
	anASClass removeFromSystem.
	(Smalltalk globals at: #TFAMIXTestClass) removeFromSystem.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateASTrait
	
	|   aTrait nbTraits |

	nbTraits := Smalltalk allTraits size.

	aTrait := aGenerator createASTrait: FAMIXTestClass .
	
	self assert: aTrait notNil.
	self assert: (aTrait name) equals: #TFAMIXTestClass.
	self assert: Smalltalk allTraits size equals: (nbTraits + 1).
	self assert: aTrait selectors size equals: TMooseEntity selectors size.
	self assert: (aTrait traitComposition allTraits includes: TMooseEntity).
	
	aTrait removeFromSystem.! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateASTraitExisting
	
	|   aTrait nbMeth nbTraits |

	nbTraits := Trait allInstances size.
	nbMeth := TFAMIXClass selectors size.

	aTrait := aGenerator createASTrait: FAMIXClass .

	self assert: aTrait notNil.
	self assert: (aTrait name) equals: #TFAMIXClass.
	self assert: Trait allInstances size equals: nbTraits.
	self assert: aTrait selectors size equals: nbMeth.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateClassName

	self assert: (aGenerator createClassName: FAMIXPackage ) equals: #ASPackage.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateClassNameNotFamix

	self assert: (aGenerator createClassName: MooseEntity ) equals: #ASMooseEntity.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateTraitName

	self assert: (aGenerator createTraitName: FAMIXPackage ) equals: #TFAMIXPackage.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:04'!
testCreateTraitNameNotFamix

	self assert: (aGenerator createTraitName: MooseEntity ) equals: #TMooseEntity.
	! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 18:00'!
testGetPropertiesOf

	self assert: (aGenerator getPropertiesOf: FAMIXTestClass ) size  equals: 1
! !

!ASGeneratorTest methodsFor: 'tests' stamp: 'AnneEtien 4/3/2013 17:51'!
testGetPropertiesOfEmpty

	self assert: (aGenerator getPropertiesOf: Object) size  equals: 0
! !

Object subclass: #ASGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrionGenerator'!

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:18'!
addAccessorFrom: aFamixClass toTrait: aOrionTrait
	"add accessors corresponding to the non heritated properties and relations of the Famix Class to the Orion trait "
	
	(self getPropertiesOf: aFamixClass) do: [ :prop | 
		aOrionTrait compile: (self codeForSelector: prop).
		aOrionTrait compile: (self codeForASSelector: prop).
		]
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:29'!
codeForASSelector: aProperty
	^((aProperty name), 'AS: aVersion
	^ self private' , (aProperty name) capitalized , ' versionIn: aVersion')
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:20'!
codeForSelector: aProperty
	^((aProperty name), '
	^ self ' , (aProperty name), 'AS: self currentContext')
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 16:07'!
createASClass: aFamixClass
	"Create an Orion Class corresponding to the FamixClass and inheriting from it"

	| name |
	name := self createClassName: aFamixClass .
	
	^Smalltalk globals at: name ifAbsent: [
		aFamixClass  subclass: name
		uses: (self createASTrait: aFamixClass)
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'OrionCore-model']! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 16:27'!
createASTrait: aFamixClass
	"Create a trait corresponding to the FamixClass "
	| name |
	name :=  self createTraitName: aFamixClass.
	^Smalltalk globals at: name ifAbsent: [
		Trait named: name
		uses: (self createASTrait: (aFamixClass superclass))
		category: 'OrionCore-model-trait'] .
	! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 13:14'!
createClassName: aFamixClass
	"Create the name of the Orion Class from the FamixClass"

	| name |

	name := aFamixClass name asString.
	name := (name beginsWith: 'FAMIX') ifTrue:  [name copyAfter: $X]
					ifFalse: [name].
	^ ('AS' , name ) asSymbol ! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 13:16'!
createTraitName: aFamixClass
	"Create the name of the Orion Class from the FamixClass"

	| name |

	name := aFamixClass name asString.
	^ ('T' , name ) asSymbol ! !

!ASGenerator methodsFor: 'as yet unclassified' stamp: 'AnneEtien 4/3/2013 17:55'!
getPropertiesOf: aFamixClass

	| metaDesc |
	metaDesc := MooseModel meta elements detect: [ :md | (md class = FM3MetaDescription ) and: [md implementingClass = aFamixClass ] ] ifNone: [ nil ].
	
	^ (metaDesc notNil)
	ifTrue: [ metaDesc attributes reject: [:a | a isDerived ]] ifFalse: [ OrderedCollection new ]! !
